package be.cylab.java.roc;

import com.opencsv.CSVWriter;


import java.io.IOException;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

/**
 * Utils class.
 */
public final class Utils {

    private Utils() {

    }

    /**
     * Count the number of positive elements (true) in a list of Point.
     *
     * @param points List<Point>
     * @return
     */
    public static int countPositiveExamples(final List<Point> points) {
        int positive_examples = 0;
        for (Point point : points) {
            if (point.getTrueAlert() == 1) {
                positive_examples++;
            }
        }
        return positive_examples;
    }

    /**
     * Count the number of negative elements (false) in a list of Point.
     *
     * @param points List<Point>
     * @return
     */
    public static int countNegativeExamples(final List<Point> points) {
        int negative_examples = 0;
        for (Point point : points) {
            if (point.getTrueAlert() == 0) {
                negative_examples++;
            }
        }
        return negative_examples;
    }

    /**
     * Method to save RocCoordinates list in a CSV file.
     *
     * @param elements List<RocCoordinates>
     * @param filepath String
     */
    public static void storeRocCoordinatesInCSVFile(
            final List<CurveCoordinates> elements,
            final String filepath) {
        try {
            FileOutputStream fos = new FileOutputStream(filepath);
            Writer output = new OutputStreamWriter(fos, StandardCharsets.UTF_8);

            CSVWriter writer = new CSVWriter(output);
            List<String[]> data = new ArrayList<>();
            for (CurveCoordinates element : elements) {
                String[] el = new String[]
                        {Double.toString(element.getXAxis()),
                                Double.toString(element.getYAxis())};
                data.add(el);
            }
            writer.writeAll(data);
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static CurveCoordinates interpolationPoint(final Statistics actual,
                                               final Statistics next) {
        double interp_recall = (next.getRecall() - actual.getRecall()) / 2;
        double x = (Math.abs(next.getTruePositiveCounter()
                - actual.getTruePositiveCounter())) / 2.0;
        double denom_denom = ((next.getFalsePositiveCounter()
                - actual.getFalsePositiveCounter())
                / (double) (next.getTruePositiveCounter()
                - actual.getTruePositiveCounter())) * x;
        if (Double.isNaN(denom_denom)) {
            denom_denom = 0.0;
        }
        double denom = actual.getTruePositiveCounter()
                + x
                + actual.getFalsePositiveCounter() + denom_denom;
        double interp_prec = (actual.getTruePositiveCounter() + x)
                / denom;
        CurveCoordinates p = new CurveCoordinates(
                actual.getRecall() + interp_recall, interp_prec);
        return p;
    }
}
